@@ -4,7 +4,7 @@ MarkdownTest = proc do
   def markdown_app(&block)
     mock_app do
       set :views, File.dirname(__FILE__) + '/views'
-      get '/', &block
+      get('/', &block)
     end
     get '/'
   end
@@ -48,7 +48,9 @@ MarkdownTest = proc do
   end
 
   it "renders with file layouts" do
-    markdown_app { markdown 'Hello World', :layout => :layout2, :layout_engine => :erb }
+    markdown_app {
+      markdown 'Hello World', :layout => :layout2, :layout_engine => :erb
+    }
     assert ok?
     assert_body "ERB Layout!\n<p>Hello World</p>"
   end
@@ -57,9 +59,7 @@ MarkdownTest = proc do
     mock_app do
       template(:inner) { "hi" }
       template(:outer) { "<outer><%= markdown :inner %></outer>" }
-      get '/' do
-        erb :outer
-      end
+      get('/') { erb :outer }
     end
 
     get '/'
@@ -74,8 +74,9 @@ Tilt.mappings['md'].each do |t|
     t.new { "" }
     klass = Class.new(Test::Unit::TestCase) { define_method(:engine) { t }}
     klass.class_eval(&MarkdownTest)
-    Object.const_set t.name[/[^:]+(?=Template$)/] << "Test", klass
-  rescue LoadError
+    name = t.name[/[^:]+$/].sub(/Template$/, '') << "Test"
+    Object.const_set name, klass
+  rescue LoadError, NameError
     warn "#{$!}: skipping markdown tests with #{t}"
   end
 end
