@@ -23,7 +23,7 @@ class RegexpLookAlike
 end
 
 class RoutingTest < Test::Unit::TestCase
-  %w[get put post delete options patch].each do |verb|
+  %w[get put post delete options patch link unlink].each do |verb|
     it "defines #{verb.upcase} request handlers with #{verb}" do
       mock_app {
         send verb, '/hello' do
@@ -70,6 +70,17 @@ class RoutingTest < Test::Unit::TestCase
     assert_equal 'pass', response.headers['X-Cascade']
   end
 
+  it "404s and does not set X-Cascade header when no route satisfies the request and x_cascade has been disabled" do
+    mock_app {
+      disable :x_cascade
+      get('/foo') { }
+    }
+    get '/bar'
+    assert_equal 404, status
+    assert_equal nil, response.headers['X-Cascade']
+  end
+
+
   it "allows using unicode" do
     mock_app do
       get('/föö') { }
@@ -100,6 +111,19 @@ class RoutingTest < Test::Unit::TestCase
     assert_equal "<h1>Not Found</h1>", response.body
   end
 
+  it "recalculates body length correctly for 404 response" do
+    mock_app {
+      get '/' do
+        @response["Content-Length"] = "30"
+        raise Sinatra::NotFound
+      end
+    }
+
+    get "/"
+    assert_equal "18", response["Content-Length"]
+    assert_equal 404, status
+  end
+
   it 'matches empty PATH_INFO to "/" if no route is defined for ""' do
     mock_app do
       get '/' do
@@ -236,6 +260,12 @@ class RoutingTest < Test::Unit::TestCase
     assert_equal "format=", body
   end
 
+  it 'does not concatinate params with the same name' do
+    mock_app { get('/:foo') { params[:foo] } }
+    get '/a?foo=b'
+    assert_body 'a'
+  end
+
   it "supports single splat params like /*" do
     mock_app {
       get '/*' do
@@ -329,6 +359,17 @@ class RoutingTest < Test::Unit::TestCase
     assert not_found?
   end
 
+  it "does not convert plus sign into space as the value of a named param" do
+    mock_app do
+      get '/:test' do
+        params["test"]
+      end
+    end
+    get '/bob+ross'
+    assert ok?
+    assert_equal 'bob+ross', body
+  end
+
   it "literally matches parens in paths" do
     route_def '/test(bar)/'
 
@@ -360,6 +401,30 @@ class RoutingTest < Test::Unit::TestCase
     assert_equal 'well, alright', body
   end
 
+  it "exposes params nested within arrays with indifferent hash" do
+    mock_app {
+      get '/testme' do
+        assert_equal 'baz', params['bar'][0]['foo']
+        assert_equal 'baz', params['bar'][0][:foo]
+        'well, alright'
+      end
+    }
+    get '/testme?bar[][foo]=baz'
+    assert_equal 'well, alright', body
+  end
+
+  it "supports arrays within params" do
+    mock_app {
+      get '/foo' do
+        assert_equal ['A', 'B'], params['bar']
+        'looks good'
+      end
+    }
+    get '/foo?bar[]=A&bar[]=B'
+    assert ok?
+    assert_equal 'looks good', body
+  end
+
   it "supports deeply nested params" do
     expected_params = {
       "emacs" => {
@@ -531,6 +596,17 @@ class RoutingTest < Test::Unit::TestCase
     assert_equal 'HelloWorldHowAreYou', body
   end
 
+  it 'sets response.status with halt' do
+    status_was = nil
+    mock_app do
+      after { status_was = status }
+      get('/') { halt 500, 'error' }
+    end
+    get '/'
+    assert_status 500
+    assert_equal 500, status_was
+  end
+
   it "transitions to the next matching route on pass" do
     mock_app {
       get '/:foo' do
@@ -692,6 +768,119 @@ class RoutingTest < Test::Unit::TestCase
     assert_equal 'Hello Bar', body
   end
 
+  it 'matches mime_types with dots, hyphens and plus signs' do
+    mime_types = %w(
+      application/atom+xml
+      application/ecmascript
+      application/EDI-X12
+      application/EDIFACT
+      application/json
+      application/javascript
+      application/octet-stream
+      application/ogg
+      application/pdf
+      application/postscript
+      application/rdf+xml
+      application/rss+xml
+      application/soap+xml
+      application/font-woff
+      application/xhtml+xml
+      application/xml
+      application/xml-dtd
+      application/xop+xml
+      application/zip
+      application/gzip
+      audio/basic
+      audio/L24
+      audio/mp4
+      audio/mpeg
+      audio/ogg
+      audio/vorbis
+      audio/vnd.rn-realaudio
+      audio/vnd.wave
+      audio/webm
+      image/gif
+      image/jpeg
+      image/pjpeg
+      image/png
+      image/svg+xml
+      image/tiff
+      image/vnd.microsoft.icon
+      message/http
+      message/imdn+xml
+      message/partial
+      message/rfc822
+      model/example
+      model/iges
+      model/mesh
+      model/vrml
+      model/x3d+binary
+      model/x3d+vrml
+      model/x3d+xml
+      multipart/mixed
+      multipart/alternative
+      multipart/related
+      multipart/form-data
+      multipart/signed
+      multipart/encrypted
+      text/cmd
+      text/css
+      text/csv
+      text/html
+      text/javascript
+      application/javascript
+      text/plain
+      text/vcard
+      text/xml
+      video/mpeg
+      video/mp4
+      video/ogg
+      video/quicktime
+      video/webm
+      video/x-matroska
+      video/x-ms-wmv
+      video/x-flv
+      application/vnd.oasis.opendocument.text
+      application/vnd.oasis.opendocument.spreadsheet
+      application/vnd.oasis.opendocument.presentation
+      application/vnd.oasis.opendocument.graphics
+      application/vnd.ms-excel
+      application/vnd.openxmlformats-officedocument.spreadsheetml.sheet
+      application/vnd.ms-powerpoint
+      application/vnd.openxmlformats-officedocument.presentationml.presentation
+      application/vnd.openxmlformats-officedocument.wordprocessingml.document
+      application/vnd.mozilla.xul+xml
+      application/vnd.google-earth.kml+xml
+      application/x-deb
+      application/x-dvi
+      application/x-font-ttf
+      application/x-javascript
+      application/x-latex
+      application/x-mpegURL
+      application/x-rar-compressed
+      application/x-shockwave-flash
+      application/x-stuffit
+      application/x-tar
+      application/x-www-form-urlencoded
+      application/x-xpinstall
+      audio/x-aac
+      audio/x-caf
+      image/x-xcf
+      text/x-gwt-rpc
+      text/x-jquery-tmpl
+      application/x-pkcs12
+      application/x-pkcs12
+      application/x-pkcs7-certificates
+      application/x-pkcs7-certificates
+      application/x-pkcs7-certreqresp
+      application/x-pkcs7-mime
+      application/x-pkcs7-mime
+      application/x-pkcs7-signature
+    )
+
+    mime_types.each { |mime_type| assert mime_type.match(Sinatra::Request::HEADER_VALUE_WITH_PARAMS) }
+  end
+
   it "filters by accept header" do
     mock_app {
       get '/', :provides => :xml do
@@ -700,6 +889,9 @@ class RoutingTest < Test::Unit::TestCase
       get '/foo', :provides => :html do
         env['HTTP_ACCEPT']
       end
+      get '/stream', :provides => 'text/event-stream' do
+        env['HTTP_ACCEPT']
+      end
     }
 
     get '/', {}, { 'HTTP_ACCEPT' => 'application/xml' }
@@ -707,7 +899,17 @@ class RoutingTest < Test::Unit::TestCase
     assert_equal 'application/xml', body
     assert_equal 'application/xml;charset=utf-8', response.headers['Content-Type']
 
-    get '/', {}, { :accept => 'text/html' }
+    get '/', {}, {}
+    assert ok?
+    assert_equal '', body
+    assert_equal 'application/xml;charset=utf-8', response.headers['Content-Type']
+
+    get '/', {}, { 'HTTP_ACCEPT' => '*/*' }
+    assert ok?
+    assert_equal '*/*', body
+    assert_equal 'application/xml;charset=utf-8', response.headers['Content-Type']
+
+    get '/', {}, { 'HTTP_ACCEPT' => 'text/html;q=0.9' }
     assert !ok?
 
     get '/foo', {}, { 'HTTP_ACCEPT' => 'text/html;q=0.9' }
@@ -715,7 +917,55 @@ class RoutingTest < Test::Unit::TestCase
     assert_equal 'text/html;q=0.9', body
 
     get '/foo', {}, { 'HTTP_ACCEPT' => '' }
+    assert ok?
+    assert_equal '', body
+
+    get '/foo', {}, { 'HTTP_ACCEPT' => '*/*' }
+    assert ok?
+    assert_equal '*/*', body
+
+    get '/foo', {}, { 'HTTP_ACCEPT' => 'application/xml' }
     assert !ok?
+
+    get '/stream', {}, { 'HTTP_ACCEPT' => 'text/event-stream' }
+    assert ok?
+    assert_equal 'text/event-stream', body
+
+    get '/stream', {}, { 'HTTP_ACCEPT' => '' }
+    assert ok?
+    assert_equal '', body
+
+    get '/stream', {}, { 'HTTP_ACCEPT' => '*/*' }
+    assert ok?
+    assert_equal '*/*', body
+
+    get '/stream', {}, { 'HTTP_ACCEPT' => 'application/xml' }
+    assert !ok?
+  end
+
+  it "filters by current Content-Type" do
+    mock_app do
+      before('/txt') { content_type :txt }
+      get('*', :provides => :txt) { 'txt' }
+
+      before('/html') { content_type :html }
+      get('*', :provides => :html) { 'html' }
+    end
+
+    get '/', {}, { 'HTTP_ACCEPT' => '*/*' }
+    assert ok?
+    assert_equal 'text/plain;charset=utf-8', response.headers['Content-Type']
+    assert_body 'txt'
+
+    get '/txt', {}, { 'HTTP_ACCEPT' => 'text/plain' }
+    assert ok?
+    assert_equal 'text/plain;charset=utf-8', response.headers['Content-Type']
+    assert_body 'txt'
+
+    get '/', {}, { 'HTTP_ACCEPT' => 'text/html' }
+    assert ok?
+    assert_equal 'text/html;charset=utf-8', response.headers['Content-Type']
+    assert_body 'html'
   end
 
   it "allows multiple mime types for accept header" do
@@ -735,21 +985,7 @@ class RoutingTest < Test::Unit::TestCase
     end
   end
 
-  it 'degrades gracefully when optional accept header is not provided' do
-    mock_app {
-      get '/', :provides => :xml do
-        env['HTTP_ACCEPT']
-      end
-      get '/' do
-        'default'
-      end
-    }
-    get '/'
-    assert ok?
-    assert_equal 'default', body
-  end
-
-  it 'respects user agent prefferences for the content type' do
+  it 'respects user agent preferences for the content type' do
     mock_app { get('/', :provides => [:png, :html]) { content_type }}
     get '/', {}, { 'HTTP_ACCEPT' => 'image/png;q=0.5,text/html;q=0.8' }
     assert_body 'text/html;charset=utf-8'
@@ -762,8 +998,6 @@ class RoutingTest < Test::Unit::TestCase
       get('/', :provides => :xml) { content_type }
       get('/') { 'no match' }
     end
-    get '/'
-    assert_body 'no match'
     get '/', {}, { 'HTTP_ACCEPT' => 'foo/*' }
     assert_body 'no match'
     get '/', {}, { 'HTTP_ACCEPT' => 'application/*' }
@@ -804,6 +1038,55 @@ class RoutingTest < Test::Unit::TestCase
     assert_body 'text/html;charset=utf-8'
   end
 
+  it 'supplies a default quality of 1.0' do
+    mock_app { get('/', :provides => [:png, :html]) { content_type }}
+    get '/', {}, { 'HTTP_ACCEPT' => 'image/png;q=0.5, text/*' }
+    assert_body 'text/html;charset=utf-8'
+  end
+
+  it 'orders types with equal quality by parameter count' do
+    mock_app do
+      get('/', :provides => [:png, :jpg]) { content_type }
+    end
+
+    lo_png = 'image/png;q=0.5'
+    hi_png = 'image/png;q=0.5;profile=FOGRA40;gamma=0.8'
+    jpeg   = 'image/jpeg;q=0.5;compress=0.25'
+
+    get '/', {}, { 'HTTP_ACCEPT' => "#{lo_png}, #{jpeg}" }
+    assert_body 'image/jpeg'
+    get '/', {}, { 'HTTP_ACCEPT' => "#{hi_png}, #{jpeg}" }
+    assert_body 'image/png'
+  end
+
+  it 'ignores the quality parameter when ordering by parameter count' do
+    mock_app do
+      get('/', :provides => [:png, :jpg]) { content_type }
+    end
+
+    lo_png = 'image/png'
+    hi_png = 'image/png;profile=FOGRA40;gamma=0.8'
+    jpeg   = 'image/jpeg;q=1.0;compress=0.25'
+
+    get '/', {}, { 'HTTP_ACCEPT' => "#{jpeg}, #{lo_png}" }
+    assert_body 'image/jpeg'
+    get '/', {}, { 'HTTP_ACCEPT' => "#{jpeg}, #{hi_png}" }
+    assert_body 'image/png'
+  end
+
+  it 'properly handles quoted strings in parameters' do
+    mock_app do
+      get('/', :provides => [:png, :jpg]) { content_type }
+    end
+
+    get '/', {}, { 'HTTP_ACCEPT' => 'image/png;q=0.5;profile=",image/jpeg,"' }
+    assert_body 'image/png'
+    get '/', {}, { 'HTTP_ACCEPT' => 'image/png;q=0.5,image/jpeg;q=0;x=";q=1.0"' }
+    assert_body 'image/png'
+    get '/', {}, { 'HTTP_ACCEPT' => 'image/png;q=0.5,image/jpeg;q=0;x="\";q=1.0"' }
+    assert_body 'image/png'
+  end
+
   it 'accepts both text/javascript and application/javascript for js' do
     mock_app { get('/', :provides => :js) { content_type }}
     get '/', {}, { 'HTTP_ACCEPT' => 'application/javascript' }
@@ -1095,4 +1378,14 @@ class RoutingTest < Test::Unit::TestCase
     assert_equal 4, signature.length
     assert list.include?(signature)
   end
+
+  it "sets env['sinatra.route'] to the matched route" do
+    mock_app do
+      after do
+        assert_equal 'GET /users/:id/status', env['sinatra.route']
+      end
+      get('/users/:id/status') { 'ok' }
+    end
+    get '/users/1/status'
+  end
 end
