@@ -1,12 +1,12 @@
-require File.dirname(__FILE__) + '/helper'
+require File.expand_path('../helper', __FILE__)
 
 class SettingsTest < Test::Unit::TestCase
   setup do
     @base = Sinatra.new(Sinatra::Base)
-    @base.set :environment, :foo
+    @base.set :environment => :foo, :app_file => nil
 
     @application = Sinatra.new(Sinatra::Application)
-    @application.set :environment, :foo
+    @application.set :environment => :foo, :app_file => nil
   end
 
   it 'sets settings to literal values' do
@@ -115,6 +115,32 @@ class SettingsTest < Test::Unit::TestCase
     assert_equal 'oops', @base.foo
   end
 
+  it 'merges values of multiple set calls if those are hashes' do
+    @base.set :foo, :a => 1
+    sub = Class.new(@base)
+    sub.set :foo, :b => 2
+    assert_equal({:a => 1, :b => 2}, sub.foo)
+  end
+
+  it 'merging does not affect the superclass' do
+    @base.set :foo, :a => 1
+    sub = Class.new(@base)
+    sub.set :foo, :b => 2
+    assert_equal({:a => 1}, @base.foo)
+  end
+
+  it 'is possible to change a value from a hash to something else' do
+    @base.set :foo, :a => 1
+    @base.set :foo, :bar
+    assert_equal(:bar, @base.foo)
+  end
+
+  it 'merges values with values of the superclass if those are hashes' do
+    @base.set :foo, :a => 1
+    @base.set :foo, :b => 2
+    assert_equal({:a => 1, :b => 2}, @base.foo)
+  end
+
   it "sets multiple settings to true with #enable" do
     @base.enable :sessions, :foo, :bar
     assert @base.sessions
@@ -222,27 +248,30 @@ class SettingsTest < Test::Unit::TestCase
     end
 
     it 'does not override app-specified error handling when set to :after_handler' do
-      klass = Sinatra.new(Sinatra::Application)
-      mock_app(klass) {
+      ran = false
+      mock_app do
         set :show_exceptions, :after_handler
-        
-        error RuntimeError do
-          'Big mistake !'
-        end
-        
-        get '/' do
-          raise RuntimeError
-        end  
-      }
-      
+        error(RuntimeError) { ran = true }
+        get('/') { raise RuntimeError }
+      end
+
       get '/'
       assert_equal 500, status
+      assert ran
+    end
 
-      assert ! body.include?("<code>")
-      assert body.include? "Big mistake !"
-      
+    it 'does catch any other exceptions when set to :after_handler' do
+      ran = false
+      mock_app do
+        set :show_exceptions, :after_handler
+        error(RuntimeError) { ran = true }
+        get('/') { raise ArgumentError }
+      end
+
+      get '/'
+      assert_equal 500, status
+      assert !ran
     end
-    
   end
 
   describe 'dump_errors' do
@@ -277,6 +306,29 @@ class SettingsTest < Test::Unit::TestCase
       get '/'
       assert body.include?("RuntimeError") && body.include?("settings_test.rb")
     end
+
+    it 'does not dump 404 errors' do
+      klass = Sinatra.new(Sinatra::Application)
+
+      mock_app(klass) {
+        enable :dump_errors
+        disable :raise_errors
+
+        error do
+          error = @env['rack.errors'].instance_variable_get(:@error)
+          error.rewind
+
+          error.read
+        end
+
+        get '/' do
+          raise Sinatra::NotFound
+        end
+      }
+
+      get '/'
+      assert !body.include?("NotFound") && !body.include?("settings_test.rb")
+    end
   end
 
   describe 'sessions' do
@@ -307,13 +359,13 @@ class SettingsTest < Test::Unit::TestCase
       assert ! @base.static?
     end
 
-    it 'is enabled on Base when public is set and exists' do
+    it 'is enabled on Base when public_folder is set and exists' do
       @base.set :environment, :development
-      @base.set :public, File.dirname(__FILE__)
+      @base.set :public_folder, File.dirname(__FILE__)
       assert @base.static?
     end
 
-    it 'is enabled on Base when root is set and root/public exists' do
+    it 'is enabled on Base when root is set and root/public_folder exists' do
       @base.set :environment, :development
       @base.set :root, File.dirname(__FILE__)
       assert @base.static?
@@ -323,17 +375,36 @@ class SettingsTest < Test::Unit::TestCase
       assert ! @application.static?
     end
 
-    it 'is enabled on Application when public is set and exists' do
+    it 'is enabled on Application when public_folder is set and exists' do
       @application.set :environment, :development
-      @application.set :public, File.dirname(__FILE__)
+      @application.set :public_folder, File.dirname(__FILE__)
       assert @application.static?
     end
 
-    it 'is enabled on Application when root is set and root/public exists' do
+    it 'is enabled on Application when root is set and root/public_folder exists' do
       @application.set :environment, :development
       @application.set :root, File.dirname(__FILE__)
       assert @application.static?
     end
+
+    it 'is possible to use Module#public' do
+      @base.send(:define_method, :foo) { }
+      @base.send(:private, :foo)
+      assert !@base.public_method_defined?(:foo)
+      @base.send(:public, :foo)
+      assert @base.public_method_defined?(:foo)
+    end
+
+    it 'is possible to use the keyword public in a sinatra app' do
+      app = Sinatra.new do
+        private
+        def priv; end
+        public
+        def pub; end
+      end
+      assert !app.public_method_defined?(:priv)
+      assert app.public_method_defined?(:pub)
+    end
   end
 
   describe 'bind' do
@@ -351,16 +422,31 @@ class SettingsTest < Test::Unit::TestCase
   end
 
   describe 'server' do
-    it 'is one of thin, mongrel, webrick' do
-      assert_equal %w[thin mongrel webrick], @base.server
-      assert_equal %w[thin mongrel webrick], @application.server
+    it 'includes webrick' do
+      assert @base.server.include?('webrick')
+      assert @application.server.include?('webrick')
+    end
+
+    it 'includes puma' do
+      assert @base.server.include?('puma')
+      assert @application.server.include?('puma')
+    end
+
+    it 'includes thin' do
+      next if RUBY_ENGINE == 'jruby'
+      assert @base.server.include?('thin')
+      assert @application.server.include?('thin')
     end
   end
 
   describe 'app_file' do
-    it 'is nil' do
-      assert_nil @base.app_file
-      assert_nil @application.app_file
+    it 'is nil for base classes' do
+      assert_nil Sinatra::Base.app_file
+      assert_nil Sinatra::Application.app_file
+    end
+
+    it 'defaults to the file subclassing' do
+      assert_equal File.expand_path(__FILE__), Sinatra.new.app_file
     end
   end
 
@@ -394,18 +480,30 @@ class SettingsTest < Test::Unit::TestCase
     end
   end
 
-  describe 'public' do
+  describe 'public_folder' do
     it 'is nil if root is not set' do
-      assert @base.public.nil?
-      assert @application.public.nil?
+      assert @base.public_folder.nil?
+      assert @application.public_folder.nil?
     end
 
     it 'is set to root joined with public/' do
       @base.root = File.dirname(__FILE__)
-      assert_equal File.dirname(__FILE__) + "/public", @base.public
+      assert_equal File.dirname(__FILE__) + "/public", @base.public_folder
 
       @application.root = File.dirname(__FILE__)
-      assert_equal File.dirname(__FILE__) + "/public", @application.public
+      assert_equal File.dirname(__FILE__) + "/public", @application.public_folder
+    end
+  end
+
+  describe 'public_dir' do
+    it 'is an alias for public_folder' do
+      @base.public_dir = File.dirname(__FILE__)
+      assert_equal File.dirname(__FILE__), @base.public_dir
+      assert_equal @base.public_folder, @base.public_dir
+
+      @application.public_dir = File.dirname(__FILE__)
+      assert_equal File.dirname(__FILE__), @application.public_dir
+      assert_equal @application.public_folder, @application.public_dir
     end
   end
 
@@ -415,4 +513,70 @@ class SettingsTest < Test::Unit::TestCase
       assert ! @application.lock?
     end
   end
+
+  describe 'protection' do
+    class MiddlewareTracker < Rack::Builder
+      def self.track
+        Rack.send :remove_const, :Builder
+        Rack.const_set :Builder, MiddlewareTracker
+        MiddlewareTracker.used.clear
+        yield
+      ensure
+        Rack.send :remove_const, :Builder
+        Rack.const_set :Builder, MiddlewareTracker.superclass
+      end
+
+      def self.used
+        @used ||= []
+      end
+
+      def use(middleware, *)
+        MiddlewareTracker.used << middleware
+        super
+      end
+    end
+
+    it 'sets up Rack::Protection' do
+      MiddlewareTracker.track do
+        Sinatra::Base.new
+        assert_include MiddlewareTracker.used, Rack::Protection
+      end
+    end
+
+    it 'sets up Rack::Protection::PathTraversal' do
+      MiddlewareTracker.track do
+        Sinatra::Base.new
+        assert_include MiddlewareTracker.used, Rack::Protection::PathTraversal
+      end
+    end
+
+    it 'does not set up Rack::Protection::PathTraversal when disabling it' do
+      MiddlewareTracker.track do
+        Sinatra.new { set :protection, :except => :path_traversal }.new
+        assert_include MiddlewareTracker.used, Rack::Protection
+        assert !MiddlewareTracker.used.include?(Rack::Protection::PathTraversal)
+      end
+    end
+
+    it 'sets up RemoteToken if sessions are enabled' do
+      MiddlewareTracker.track do
+        Sinatra.new { enable :sessions }.new
+        assert_include MiddlewareTracker.used, Rack::Protection::RemoteToken
+      end
+    end
+
+    it 'does not set up RemoteToken if sessions are disabled' do
+      MiddlewareTracker.track do
+        Sinatra.new.new
+        assert !MiddlewareTracker.used.include?(Rack::Protection::RemoteToken)
+      end
+    end
+
+    it 'sets up RemoteToken if it is configured to' do
+      MiddlewareTracker.track do
+        Sinatra.new { set :protection, :session => true }.new
+        assert_include MiddlewareTracker.used, Rack::Protection::RemoteToken
+      end
+    end
+  end
 end
